#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"

#########################################################################
# winmmucl - Mark's MUd Client with a text widget to display output.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

# Edit the below paths or let make do it. 
# Do NOT edit anything else unless you know tcl/tk.
# Do NOT delete any of the preceding lines. Make depends on them.
set Mmucl(lib_dir) ..
set Mmucl(rc_dir) ..
###########################

# Make sure the paths are absolute
if {[file pathtype Mmucl(lib_dir)]=="relative"} {
    set Mmucl(lib_dir) [file join [pwd] $Mmucl(lib_dir)]
}
if {[file pathtype $Mmucl(rc_dir)]=="relative"} {
    set Mmucl(rc_dir) [file join [pwd] $Mmucl(rc_dir)]
}

# Make the package mechanism look in the right place
set auto_path [linsert $auto_path 0 $Mmucl(lib_dir)]

package require mmucl
package require mmuclGui
package require balloon

# Use provided html browser as default.
set Mmucl(browser) "[info nameofexecutable] [file join [file dirname [info script]] html_browse.tcl] -- %1"

proc echo {str} {
    .display configure -state normal
 
    .display insert end $str\n
    .display yview end  
    
    if {[.display index end]>300} {
	.display delete 0.0 50.0
    }
    .display configure -state disabled
}

# ansi color support
# Split $str into a list. Each element of the list will be
# ansi codes then plain text.

proc display {str} {
    upvar \#0 Winmmucl(ansi_color) ansi

   .display configure -state normal
    if {$ansi} {
        set tags {"" 37 40}
        foreach part [split \[37\;40m$str ] {
            regexp {^\[([0-9;]+)m(.*)} $part x codes text
            foreach code [split $codes \;] {
                if {$code<2} {
                    set tags [lreplace $tags 0 0 $code]
                } elseif {$code<38} {
                    set tags [lreplace $tags 1 1 $code]
                } else {
                    set tags [lreplace $tags 2 2 $code]
                }
            }
            .display insert end $text $tags
        }
    } else {
        .display insert end $str\n
    }    
    .display see end  
    if {[.display index end]>300} {
        .display delete 0.0 50.0
    }
    .display configure -state disabled
}

proc set_font {family size} {
    global Winmmucl
    
    if {[info commands font]!=""} {
	set normal [list $family $size normal]
	set bold [list $family $size bold]
    } else {
	set normal "-*-$family-Medium-R-Normal-*-$size-*"
	set bold "-*-$family-Bold-R-Normal-*-$size-*" 
    }
    catch {
	.display configure -font $normal
	.display tag configure 0 -font $normal
	.display tag configure 1 -font $bold
	.display tag raise sel
    }
    echo "** Font set to [.display cget -font] **"
    set Winmmucl(font_family) [lindex [.display cget -font] 0]
    set Winmmucl(font_size) [lindex [.display cget -font] 1]
}

text .display -width 70 -height 30 -wrap word -selectbackground gray \
    -yscrollcommand {.vscroll set} -background black -foreground white
scrollbar .vscroll -orient vertical -command {.display yview}

source [file join $Mmucl(lib_dir) lib mmuclGui_init]

# Make a menu
frame .menu 
menubutton .menu.file -text "File" -menu .menu.file.m 
menu .menu.file.m -tearoff 0

.menu.file.m add command -label "Connect" -command dialog_connect
.menu.file.m add separator
.menu.file.m add command -label "Load File" \
    -command {tkOpen "Load a file" source}
.menu.file.m add command -label "Save" \
    -command {tkSave "Save Current Aliases and Actions" save}
.menu.file.m add separator
.menu.file.m add command -label "Exit" -command \
    {tkDialog Mmucl "Really Quit?" exit}

menubutton .menu.help -text "Help" -menu .menu.help.m 
menu  .menu.help.m -tearoff 0

.menu.help.m add command -label "About" -command {help about}
.menu.help.m add command -label "Contents" -command help

menubutton .menu.edit -text "Edit" -menu .menu.edit.m 
menu .menu.edit.m -tearoff 0
.menu.edit.m add command -label Copy -command {tk_textCopy .display}
.menu.edit.m add command -label Paste -command {tk_textPaste .input}
.menu.edit.m add separator
.menu.edit.m add command -label "Edit Aliases" -command {dialog_alias}
.menu.edit.m add command -label "Edit Actions" -command {dialog_action}
.menu.edit.m add command -label "Edit Characters" -command {dialog_character}
.menu.edit.m add command -label "Edit Key Bindings" -command {dialog_bind}

menubutton .menu.option -text "Options" -menu .menu.option.m 
menu .menu.option.m -tearoff 0
.menu.option.m add radiobutton -label {Ansi color on} -value 1 \
    -variable Winmmucl(ansi_color)
.menu.option.m add radiobutton -label {Ansi color off} -value 0 \
    -variable Winmmucl(ansi_color)

# If font command is present...
if {[info commands font]!=""} {
    .menu.option.m add separator
    .menu.option.m add cascade -label "Font family" -menu .menu.option.m.font
    menu .menu.option.m.font -tearoff 0 
    foreach i [lsort [font families]] {
	.menu.option.m.font add radiobutton -label $i -value $i \
	    -variable Winmmucl(font_family) \
	    -command {set_font $Winmmucl(font_family) $Winmmucl(font_size)}
    }
    .menu.option.m add cascade -label "Font size" -menu .menu.option.m.size
    menu .menu.option.m.size -tearoff 0
    foreach i {6 8 10 11 12 14 18} {
	.menu.option.m.size add radiobutton -label $i -value $i \
	    -variable Winmmucl(font_size) \
	    -command {set_font $Winmmucl(font_family) $Winmmucl(font_size)}
    }
    unset i
}

pack .menu.file -side left
pack .menu.edit -side left
pack .menu.option -side left
pack .menu.help -side right

grid .menu -row 0 -column 0 -columnspan 2 -sticky "ew"
grid .tools -row 1 -column 0  -sticky "ew" -pady 1
grid .display -row 2 -column 0 -sticky "news"
grid .vscroll -row 2 -column 1 -sticky "ns"
grid .input -row 3 -column 0 -columnspan 1 -sticky "ew"
grid columnconfigure . 0 -weight 1
grid rowconfigure . 2 -weight 1
wm title . Winmucl

# Ansi color support:
# set_font does these
#.display tag configure 0 -font normal
#.display tag configure 1 -font bold
.display tag configure 4  -underline 1 
#.display tag configure 5 flashing text?
.display tag configure 30 -foreground black
.display tag configure 31 -foreground red
.display tag configure 32 -foreground green3
.display tag configure 33 -foreground yellow
.display tag configure 34 -foreground blue
.display tag configure 35 -foreground magenta
.display tag configure 36 -foreground cyan
.display tag configure 37 -foreground gray
.display tag configure 40 -background black
.display tag configure 41 -background red
.display tag configure 42 -background green3
.display tag configure 43 -background yellow
.display tag configure 44 -background blue
.display tag configure 45 -background magenta
.display tag configure 46 -background cyan
.display tag configure 47 -background gray

# Make sure selection is visible
.display tag raise sel

# Default is ansi color on.
set Winmmucl(ansi_color) 1

echo "Mmucl version 1.0b3\n"
